      subroutine eqlib                                                   
c******************************************************************************
c     This routine solves the equations of molecular dissociative           
c     equilibrium. the equations can include neutral and ionized molecules  
c     and atoms 
c******************************************************************************

      implicit real*8 (a-h,o-z)
      include 'Atmos.com'
      include 'Mol.com'
      include 'Quants.com'
      include 'Eqdata.com'
      dimension ident(10,40),xfic(10),xcorr(10),deltax(10),c(10,10),     
     1 ans(10,10)                                                        


c*****start by deciding if molecular equilibrium is desired
      if (ipr(2) .le. 0) return


c*****clear the arrays 
      iorder(1) = 0                                                
      neq = iorder(1) 
      tdel = (t(ntau)-t(1))/4.0 + 1.0                                    
      if (ipr(2) .lt. 1) return                                           
      do k=1,10                                                       
         iorder(k) = 0                                                      
         xcorr(k) = 0.                                                      
         do kk=1,40                                                      
            ident(k,kk) = 0                                                    
         enddo
      enddo


c*****read in the dissociation data                                         
      do jmol=1,nmol                                                   
         if (amol(jmol) .ge. 100.) then
            do k=1,100                                                       
               if (datmol(1,k) .eq. amol(jmol)) go to 11                   
            enddo
            write (nwrite,1001) amol(jmol)                   
            write (*,1001) amol(jmol)                   
1001        format ('I do not know this molecule: ',f10.0)                    
            stop                                                               
11          do kk=1,6                                                       
               const(kk,jmol) = datmol(kk+1,k)                            
            enddo 
         else
            iatom1 = amol(jmol)                                                
            const(1,jmol) = xchi1(iatom1)                                      
            do m=1,5                                                        
               ti = t(1) + (m-1)*tdel                                    
               atom = iatom1                                            
               it = ti                                                      
               u1 = partfn(atom,it)                                 
               u2 = partfn(atom+0.1,it)                                   
               const(m+1,jmol) = u2/u1                             
            enddo 
         endif
      enddo
      if (ipr(2) .ge. 2) call out (5)                                      


c*****set up the molecular equilibrium array. each element of array         
c*****'ident' contains the identifier (subscript of an element of           
c*****array 'amol') of an ion or a molecule. the neutral atom is always     
c*****understood, but is not explicitly contained in 'ident'.               
      nmax = 1                                                           
      do jmol=1,nmol                                                  
         atom = amol(jmol)                                                  
2        call sunder(atom,iatom1,iatom2)                                    
         do k=1,10                                                        
            if (iatom1 .eq. iorder(k)) go to 4                                  
         enddo
         neq = neq + 1                                                      
         iorder(neq) = iatom1                                               
         ident(neq,1) = jmol                                                
         go to 6                                                            
4        do kk=1,nmax                                                     
            if (ident(k,kk).eq.0 .or. ident(k,kk).eq. jmol) go to 7             
         enddo 
         nmax = nmax + 1                                                 
         kk = nmax                                                
7        ident(k,kk) = jmol                                                 
6        if (iatom2 .ne. 0) then
            atom = iatom2                                                      
            go to 2                                                            
         endif
      enddo                                                           
      if (ipr(2) .ge. 2) call out (6)                                      


c*****now begin the loop that goes through all the atmosphere tau layers
      do 21 kev=1,ntau                                                   


c*****calculate *xfic* and make a first guess at *xatom*                    
      i = ntau + 1 - kev                                           
      lev = i
      tk = 1.38054d-16*t(i)                                              
      tkev = 1.38054d-16*t(i)*6.242d11                                   
      do k=1,neq                                                      
         korder = iorder(k)                                                 
         xfic(k) = xabund(korder)*nhtot(i)                                  
      enddo
      if (i .lt. ntau) then
         do k=1,neq                                                      
            xatom(k) = xatom(k)*nhtot(i)/nhtot(i+1)                            
         enddo
      else
         do k=1,neq                                                      
            xatom(k) = xfic(k)                                                 
         enddo
      endif


c*****compute the number densities of ions and molecules                    
27    do jmol=1,nmol                                                  
         atom = amol(jmol)                                                  
         if (atom.ge.100. .and. t(i).gt.12000.) then
            xmol(jmol) = 1.0d-20
         elseif (atom.ge.100. .and. t(i).le.15000.) then
            xmol(jmol) = 1.                                                    
            count = 0.                                                         
37          call sunder(atom,iatom1,iatom2)                                    
            count = count + 1.                                                 
            do k=1,neq                                                      
               if (iorder(k) .eq. iatom1) 
     .            xmol(jmol) = xmol(jmol)*xatom(k)                       
            enddo                                                           
            if (iatom2 .ne. 0) then
               atom = iatom2                                          
               go to 37                                         
            endif
            hion = 10.0*(amol(jmol) - dint(amol(jmol)))                        
            xmol(jmol) = xmol(jmol)*dexp(const(1,jmol)/tkev - 
     .                   const(2,jmol) + (const(3,jmol) + 
     .                   (-const(4,jmol) + (const(5,jmol) -               
     .                   const(6,jmol)*t(i))*t(i))*t(i))*t(i) - 
     .                   1.5*(count-1.0-hion)*
     .                   dlog(t(i)))/ne(i)**hion    
         else
            delt = (t(i)-t(1))/tdel                               
            m = min0(idint(delt)+2,5)                          
            delt = delt - idint(delt)                           
             u1 = const(m,jmol) + 
     .           (const(m+1,jmol)-const(m,jmol))*delt          
            iatom1 = atom                                           
            do k=1,neq                                                      
               if (iorder(k) .eq. iatom1) xmol(jmol) = 
     .            4.825d15*u1*t(i)**1.5/ne(i)*dexp(-1.1605d4*            
     .            const(1,jmol)/t(i))*xatom(k)                              
            enddo
         endif
      enddo

c*****compute matrix *c*, which is the derivative of each equation with     
c*****respect to each atom.                                                 
26    do k=1,neq                                                      
         deltax(k) = -xfic(k) + xatom(k)                                    
         do kk=1,neq                                                     
            c(k,kk) = 0.                                                       
         enddo
         c(k,k) = 1.                                                        
         korder = iorder(k)                                                 
         do kk=1,neq                                                     
            kderiv = iorder(kk)                                                
            do 28 j=1,nmax                                                     
               jmol = ident(k,j)                                              
               if (jmol .eq. 0) go to 28                                     
               call discov(amol(jmol),kderiv,num2)                            
               if (num2 .eq. 0) go to 28                                        
               call discov(amol(jmol),korder,num1)                              
               c(k,kk) = c(k,kk) + xmol(jmol)*num1*num2/xatom(kk)            
               if (k .eq. kk) deltax(k) = deltax(k) + num1*xmol(jmol)          
28          continue                                                           
         enddo
      enddo

c*****calculate array 'xcorr', the change in 'xatom'. array 'xcorr' is      
c*****'deltax' multiplied by the inverse of 'c'                            
      call invert (neq,c,ans,10)                                          
      do k=1,neq                                                      
         x1 = xcorr(k)                                                      
         xcorr(k) = 0.                                                      
         do kk=1,neq                                                     
            xcorr(k) = xcorr(k) + ans(k,kk)*deltax(kk)                         
         enddo
      enddo

c*****decide if another iteration is needed                                 
      iflag = 0                                                          
      do k=1,neq                                                      
c*****here oscillations are damped out                                      
         if (x1*xcorr(k) .lt. -0.5*x1**2) xcorr(k) = 0.5*xcorr(k)            
         x1 = xatom(k)                                                      
         if (dabs(xcorr(k)/xatom(k)) .gt. 0.005) iflag = 1                    
         xatom(k) = xatom(k) - xcorr(k)                                     
c*****fix element number densities which are ridiculous                     
         if (xatom(k).le.0.0 .or. xatom(k).ge.1.001*xfic(k)) then
            iflag = 1                                                          
            xatom(k) = 1.0d-2*dabs(xatom(k)+xcorr(k))                           
         endif
      enddo                                                           
      if (iflag .ne. 0) go to 27                                          

c*****print out atomic and molecular partial pressures. *xamol* is the      
c*****number density for each neutral atom                                  
      do k=1,neq                                                     
         xamol(k,i) = xatom(k)                                              
         xatom(k) = dlog10(xatom(k)*tk)                                     
      enddo
      do jmol=1,nmol                                                 
         xmol(jmol) = dlog10(xmol(jmol)*tk)                                 
      enddo
      if (ipr(2) .ge. 2) call out (7)                                      
      do k=1,neq                                                     
         xatom(k) = 10.0**xatom(k)/tk                                       
      enddo

c*****here the big loop in tau ends
21    continue                                                           


      return                                                             
      end                                                                



